// !Headings v1.06

// Produces page headings based on Headings in the main text.
// Written by Roger Noble, August 2000

// Version History:
// v1.0 (11th August 2000)  :  Initial version
// v1.01 (19th August 2000) :  Fixed fault with maintaining lowest bookmark.
//                             Added support for specifying separate styles for
//                             left and right pages.
//                             Added Find button and support.
// v1.02 (23rd August 2000) :  Fixed error in find_bookmark which meant that it did not
//			       work. Funny, I thought that I had tested it ...
// v1.03 (14th Sept 2000)   :  Fixed problem that resulted in last headings not being done
//			       under some circumstances.
// v1.04 (24th Nov 2000)    :  Removed a few legacy variables ...
// v1.05 (6th April 2001)   :  Display hour glass when busy.
// v1.06 (20th July 2001)   :  Use 7!bm if present to hide our bookmarks.

//=============================================================================================

// Global Variables.

// Options.

int Headings_Alternate = 1;    // Alternate headings from side to side
int Headings_AlternateTemp;    // Used to hold values whilst Headings dialogue active
int Headings_RemoveDot = 1;    // Remove full stop after heading
int Headings_RemoveDotTemp;    // Used to hold values whilst Headings dialogue active
int Headings_RemoveNum = 1;    // Remove heading numbers
int Headings_RemoveNumTemp;    // Used to hold values whilst Headings dialogue active
string Headings_Separator;     // Separator for numbers and headings

string Headings_ActualSep;     // Actual number separator to use


// Handles and techie things.

int Headings_stylemenu;     // Style menu-button menu handle
int Headings_win;           // The options window handle

// Stuff to do with the headings bookmarks.

int Headings_high_water = 1;	// Highest headings bookmark number.
int Headings_low_water = 1;	// Lowest headings bookmark number.

int Headings_current_bm = 0;    // Current headings bookmark when finding

// Styles.

int Headings_Style1 = -1;   // Style handles
int Headings_Style2 = -1;
int Headings_Style3 = -1;
int Headings_Style4 = -1;
string Headings_Style1s;    // Style names
string Headings_Style2s;
string Headings_Style3s;
string Headings_Style4s;
int Headings_temp1 = -1;    // Used in Headings dialogue to hold style handles
int Headings_temp2 = -1;
int Headings_temp3 = -1;
int Headings_temp4 = -1;

// Window icons.

int Headings_current_icon = 0; // Which style am I currently editing
                               // Bad style to have it global, but this is the simplest
                               // way of getting the job done
// Document-related stuff.

int Headings_current_file = 0; 			       // Handle to current file
int Headings_current_view = 0;			       // Handle to current view

// Constants.

int Headings_tag = ('h'<<24) | ('e'<<16) | ('a'<<8);   // Base tag value
int Headings_truncate = 512;			       // Truncate a heading if longer than this

//=============================================================================================

////////////////////////////////////////////////
// Headings Page Producing Functions          //
////////////////////////////////////////////////

// Test if the style is one of the styles for a heading. Returns the
// style number, or zero if not.

int Headings_isstyle()
{
  caretcontext();

  iscurrentstyle(0);			      // Set up styles

  if (iscurrentstyle(Headings_Style1) != 0)   // Is it style 1 at the caret?
    return(1);                                // If so return 1
  if (iscurrentstyle(Headings_Style2) != 0)   // and so on
    return(2);
  if (iscurrentstyle(Headings_Style3) != 0)
    return(3);
  if (iscurrentstyle(Headings_Style4) != 0)
    return(4);

  return(0);                       // It wasn't any of the known styles
}

//---------------------------------------------------------------------------------------------

// Clear all the styles from the selected or current text.

int Headings_clearallstyles()
{
 int i = 0;
 string s;

 while(getcurrentstyle(i, s, 2) != 0)
 {
  remstyle(i++);
 }
 return(0);
}

//---------------------------------------------------------------------------------------------

// Find a style with the name "stylename", return the handle for it. Returns -1
// if the style is not found.

int Headings_findstyle(string stylename)
{
  int temp;
  string tempstring;

  for (temp = 0; getuserstyle(temp, tempstring, 0) != 0; temp++)
  {
    if (tempstring == stylename)
      return(temp);
  }

  return -1;
}

//---------------------------------------------------------------------------------------------

// Sets the style handles from the style names.

void Headings_set_styles()
{
    Headings_Style1 = Headings_findstyle (Headings_Style1s);
    Headings_Style2 = Headings_findstyle (Headings_Style2s);
    Headings_Style3 = Headings_findstyle (Headings_Style3s);
    Headings_Style4 = Headings_findstyle (Headings_Style4s);
    Headings_temp1 = Headings_Style1;
    Headings_temp2 = Headings_Style2;
    Headings_temp3 = Headings_Style3;
    Headings_temp4 = Headings_Style4;
}

//---------------------------------------------------------------------------------------------

// Parses the separator string provided and returns a string suitable for direct
// comparison with the in-line text.

void Headings_parse_separator(string& separator)
{
  int separator_len = slen(Headings_Separator);
  int i = 0;
  int c;

  separator = "";

  while (i < separator_len)
  {
    c = schar(Headings_Separator, i);

    if (c == '\\')
    {
      // Quoted character, check the next one.

      if (i+1 < separator_len)
      {
        c = schar(Headings_Separator, i+1);

        switch (c)
        {
          case 't':    // Tab
            separator += chars(9);
            break;

          case 'r':   // Newlines and the like, replace with space
          case 'n':
            separator += " ";
            break;

          default:    // Others, replace with the character following the '\'
            separator += chars(c);
            break;
        }

        i += 2;
      }
    }
    else if (c >= 32)
    {
      separator += chars(c);   // Normal character
      ++i;
    }
    else
      ++i;                     // Ignore control characters
  }
}

//---------------------------------------------------------------------------------------------

// Scans through a chapter writing a temporary file containing the main text headings
// located and their page numbers. Returns 0 if there are more chapters (stories) to
// do.

int Headings_styletest (int tempbm, int temp_file)
{
  int level;
  int i = 1;
  int end;
  int bm;
  int bm1;
  int bm2;
  int c;
  int chapter;
  int numberformat;

  bm = bmcreate ("Headings_SBM");        // Bookmark for going through current story
  setbmtobm(bm, tempbm);
  bm1 = bmcreate ("Headings_SBM1");	 // Bookmark for going through story
  bm2 = bmcreate ("Headings_SBM2");	 // Bookmark for checking

  chapter = bmchapter(tempbm);

  while (i != 0)                         // While not the end of text
  {
    setcarettobm(bm);                    // Move the caret to the bookmark
    setbmtobm(bm1,bm);			 // Bookmark at start of selected text
    i = bmmove(bm, 1, 3);             	 // Move bm to end of paragraph
    setzonetobm(bm);                 	 // Select the bit between caret & bm
    level = -1;
    level = Headings_isstyle();          // Find out what style it is, and
                                         // do I need it

    if (cancopy() != 0 && level != 0)    // If I can copy this selection and
                                         // its a style I need
    {
      // Got something to copy to the headings file.

      clearselection();

      // First write the heading level, chapter and page numbers.

      filewritei(level, temp_file);
      filewritei(chapter, temp_file);
      filewritei(bmpage(bm)+pagestart(chapter, currentfile()), temp_file);

      // Now the heading text.

      do
      {
        c = bmchar(bm1);

        // Test for special characters.

	switch (c)
	{
	  case 18:			 // Tab
	    fileputc ('\t', temp_file);
	    break;
	  case 4:			// Current chapter number
	    filewrites (itos(chapter), temp_file);
	    break;
	  case 5:			// Current date
	    filewrites ("{activedate}", temp_file);
	    break;
	  case '<':			// <, replace with |< to avoid system variable expansion
	    filewrites ("|<", temp_file);
	    break;
	  case '{':			// {, replace with |{ to avoid macro expansion
	    filewrites ("|{", temp_file);
	    break;
	  default:			// Other characters
	    if (c >= 32)
	      fileputc (c, temp_file);
	    break;
	}
	setbmtobm (bm2, bm1);;			// Remember where we are
        end = bmmove(bm1, 1, 0);         	// Next character
	if (bmcmp(bm1,bm2) <= 0) end = 0;	// If we did not move, finish
      }
      while (end != 0 && bmcmp(bm1,bm) < 0)
      fileputc ('\n', temp_file);	 // To terminate text
    }

    // Next paragraph.

    setbmtobm (bm2, bm);			// Remember where we are
    i = bmmove(bm, 1, 7);       		// Move down a paragraph
    if (i != 0 && bmcmp(bm,bm2) <= 0) i = 0;	// If we did not move, finish story
  }

  bmdelete (bm2);
  bmdelete (bm1);
  bmdelete (bm);

  i = bmnextstory(tempbm);		 // Move to next story

  return i;
}

//---------------------------------------------------------------------------------------------

// Locates a heading from our temporary file that is suitable for use for the supplied
// page and chapter number. Returns the level of the heading (0 if none found), and
// the heading string.

int Headings_find_heading (int chapter, int page, string& heading, int temp_file)
{
  int best_level = 0;
  string best_heading;
  string current_heading;
  int current_level;
  int current_page;
  int current_chapter = -1;

  fileseek (0, temp_file);		// Reposition temporary file

  // First locate a heading for the right chapter.

  while (!fileeof(temp_file))
  {
    current_level = filereadi(temp_file);

    if (!fileeof(temp_file))
    {
      current_chapter = filereadi(temp_file);
      current_page = filereadi(temp_file);
      filereads(best_heading, temp_file);

      if (current_chapter == chapter) break;
    }
  }

  if (current_chapter == chapter)
  {
    // Found a heading in the chapter. This will do at first.

    if (current_page == page)
    {
      heading = best_heading;  // Found it already
      return current_level;
    }

    best_level = current_level;


    // Now search for any heading on the same page. Keep updating the
    // best level and heading.

    while (!fileeof(temp_file))
    {
      current_level = filereadi(temp_file);
      if (!fileeof(temp_file))
      {
        current_chapter = filereadi(temp_file);
        current_page = filereadi(temp_file);
        filereads(current_heading, temp_file);

        if (current_chapter == chapter)
        {
          if (current_page < page)
          {
            best_level = current_level;
            best_heading = current_heading;
          }
          else if (current_page == page)
          {
            best_level = current_level;
            best_heading = current_heading;
            break;
          }
          else
            break;
        }
        else
          break;
      }
    }
  }

  heading = best_heading;

  return best_level;
}

//---------------------------------------------------------------------------------------------

// Locate a heading bookmark in the paragraph containing the cursor.
// Returns the handle of the bookmark if found, or zero it not.

int Headings_find_bookmark()
{
  int bms = bmcreate("Headings_Start");
  int bme = bmcreate("Headings_End");
  int bm;
  int i;

  setbmtocaret(bms);
  setbmtobm(bme, bms);
  bmmove(bms, 0, 3);  // Start of paragraph
  bmmove(bme, 1, 3);  // End of paragraph

  for (i = Headings_low_water; i < Headings_high_water; i++)
  {
    bm = bmfind("HeaD" + itos(i));
    if (bm)
      if (bmcmp(bms, bm) <= 0 && bmcmp(bme, bm) >= 0)
      {
        bmdelete(bms);
        bmdelete(bme);
        return bm;
      }
  }

  bmdelete(bms);
  bmdelete(bme);

  return 0;
}

//---------------------------------------------------------------------------------------------

// Locate the next headings bookmark and move the caret to it.

void Headings_next_location()
{
  int bm = 0;

  if (Headings_current_bm < Headings_low_water)
    Headings_current_bm = Headings_low_water;

  // Locate the next headings bookmark.

  while (!bm && Headings_current_bm < Headings_high_water)
  {
    bm = bmfind("HeaD" + itos(Headings_current_bm));
    Headings_current_bm++;
  }

  if (bm)
  {
    // Found one, move to it.

    setcarettobm(bm);
  }
  else
  {
    // Did not find one, bell and reset the current location.

    bbc_vdu(7);
    Headings_current_bm = 0;
  }
}

//---------------------------------------------------------------------------------------------

// Do the page headings. Here we scan though the headings bookmarks. When we locate one
// we find which page it is on and then search the temporary file for a suitable heading
// to use.

void Headings_write_headings (int temp_file)
{
  int truncate_count= 0;
  int current;
  string bookmark;
  int bm;
  int bm1;
  int level;
  int lowest = Headings_high_water;
  int highest = 0;
  int page;
  int chapter;
  int len;
  string heading;
  int hstart;
  int lens;
  string style_postfix;
  int style;

  bm1 = bmcreate("Headings_Temp");

  // Go through all possible headings bookmarks.

  for (current = Headings_low_water; current < Headings_high_water; current++)
  {
    bookmark = "HeaD" + itos(current);	// Current bookmark
    bm = bmfind(bookmark);
    if (bm)
    {
      // Found it. Move to its position, and set bookmarks at the beginning and end of the
      // existing heading.

      setbmtobm(bm1,bm);
      bmmove(bm1, 0, 3);
      bmmove(bm, 1, 3);
      setcarettobm(bm1);
      setzonetobm(bm);

      // Remove any existing effects and styles and then any existing text.

      cleareffects(0);
      Headings_clearallstyles();
      if (candelete()) deleteselection();

      // Locate a suitable heading from our temporary file.

      chapter = bmchapter(bm);
      page = bmpage(bm) + pagestart(chapter, currentfile());
      level = Headings_find_heading(chapter, page, heading, temp_file);
      len = slen(heading);

      if (len > 1)
      {
        // Are we doing different styles for left and right pages?

        if (Headings_Alternate && (chapterbits(chapter, currentfile()) & 2))
        {
          {
            if (chapterbits(chapter, currentfile()) & 1)
            {
              if (page / 2 * 2 != page)
                style_postfix = "l";
              else
                style_postfix = "r";
            }
            else
            {
              if (page / 2 * 2 == page)
                style_postfix = "l";
              else
                style_postfix = "r";
            }
          }
        }
        else
          style_postfix = "";

        // Add the appropriate headings style.

        if (level)
        {
          style = Headings_findstyle("Headings" + itos(level) + style_postfix);
          if (style < 0)
          {
            // No such style, if we are alternating headings try to find a style without
            // the postfix.

            if (slen(style_postfix) > 0)
            {
              style = Headings_findstyle("Headings" + itos(level));
              addstyle(style);

              // Apply the left or right align manually assuming that the left pages
              // need a left align and right pages need a right align.

              if (style_postfix == "l")
                setalign(0);
              else
                setalign(2);
            }
            else
              addstyle(style);
          }
          else
            addstyle(style);

          // Write the heading missing out the \n at its end and optionally any . as well.

          if (mids(heading, len-2, 1) == ".") len--;

          // Are we to remove section numbers?

          if (Headings_RemoveNum)
          {
            // Yes, search for the number separator.

            lens = slen(Headings_ActualSep);
            hstart = 0;
            while (hstart < len-lens)
            {
              if (mids(heading, hstart, lens) == Headings_ActualSep) break;
              hstart++;
            }
            if (hstart < len-lens)
            {
              hstart += lens;
              len -= hstart;
            }
            else
              hstart = 0;
          }
          else
            hstart = 0;

          // Now we can write the heading, truncating if necessary.

          if (len-hstart > Headings_truncate+1)
          {
            len = Headings_truncate + 1 - hstart;
            truncate_count++;
          }
          type(mids(heading,hstart,len-1));
        }
      }

      // Adjust the lowest and highest bookmarks actually found.

      if (current < lowest) lowest = current;
      if (current > highest) highest = current;
    }
  }

  // Adjust the bookmark high and low water marks to what actually exists.

  Headings_low_water = lowest;
  Headings_high_water = highest + 1;

  // Did we truncate any headings?

  if (truncate_count > 0)
  {
    string s = "{HEADINGS_11}";
    translate(s);
    messagebox(itos(truncate_count) + s);
  }

  bmdelete(bm1);


}

//---------------------------------------------------------------------------------------------

// Produce the headings.

void Headings_do_headings (int tempbm)
{
  int    temp_file;
  int    Headings_scrap;
  string Headings_file;

  Headings_set_styles();

  Headings_scrap=scrapcreate();
  scrapname(Headings_scrap,Headings_file);

  temp_file=fileopen(Headings_file, "wb+");
  if (temp_file == 0)
  {
    string s = "{HEADINGS_07}";
    translate(s);
    messagebox(s);
  }
  else
  {
//    swi13(0x406c0, 0, 0, 0);
    while (!Headings_styletest(tempbm, temp_file));
    Headings_write_headings(temp_file);
    fileclose(temp_file);
    systems("Del " + Headings_file);		// Delete scratch file
    bmdelete (tempbm);
//    swi13(0x406c1, 0, 0,0 );
  }

 scrapremove(Headings_scrap);
}

//=============================================================================================

////////////////////////////////////////////////
//  Headings Style Menu Functions             //
////////////////////////////////////////////////

// Checks if any headings styles have been set and that we are likely to have
// some headings bookmarks set.
// Returns 1 if any styles have been set, 0 otherwise.

int Headings_check_ready()
{

  if ((Headings_temp1 <= -1 && Headings_temp2 <= -1 && Headings_temp3 <= -1
      && Headings_temp4 <= -1) || Headings_high_water <= 1)
  {
    // No headings styles are set or there are no bookmarks.
    // Disable the Do Headings action.

    set_icon_state (Headings_win, 0, 0x00400000, 0x00400000);

    return 0;
  }

  // We can do some headings. Enable the Do Headings action.

  set_icon_state (Headings_win, 0, 0x00000000, 0x00400000);

  return 1;
}

//---------------------------------------------------------------------------------------------

int Headings_choosestyle()
{
  int i = 0;
  string tempstring = "";

  if(Headings_stylemenu)
  {
    delete_menu(Headings_stylemenu);
    Headings_stylemenu = create_menu("{HEADINGS_05}");
  }

  // Add all the menu entries.

  addentry_menu (Headings_stylemenu, "Headings_styleentry", "Headings_styleflags", "", "", "Clear Style");

  for(i=0; getuserstyle(i, tempstring, 0) != 0; i++)
  {
    addentry_menu(Headings_stylemenu, "Headings_styleentry", "Headings_styleflags", "", "", tempstring);
  }
}

//---------------------------------------------------------------------------------------------

int Headings_styleentry (int n, int subcode)
{
  string stylename;

  if (n > 0)
    getuserstyle (n-1, stylename, 0);
  else
    stylename = "";		// Clear style
  writeicon (Headings_win, (Headings_current_icon-4+100), stylename);

  switch (Headings_current_icon)
  {
    case 6:
      Headings_temp1 = n-1;
      break;

    case 7:
      Headings_temp2 = n-1;
      break;

    case 8:
      Headings_temp3 = n-1;
      break;

    case 9:
      Headings_temp4 = n-1;
      break;

    default:
      return (0);
      break;
  }

  Headings_check_ready();	// Check if ready to do headings now

  return (1);
}

//---------------------------------------------------------------------------------------------

// Tick, shade etc the items appropriately.

int Headings_styleflags (int n, string &s)
{
  int bits = 0;

  switch (Headings_current_icon)   // Find out which icon we are editing
  {
    case 6:
      if(Headings_temp1 == n-1)    // is the current style = this menu item
        bits = TICKED;     // if so then tick it
      break;

    case 7:
      if(Headings_temp2 == n-1)    // and so on...
        bits = TICKED;
      break;

    case 8:
      if(Headings_temp3 == n-1)
        bits = TICKED;
      break;

    case 9:
      if(Headings_temp4 == n-1)
        bits = TICKED;
      break;

    default:
      return (0);
      break;
  }

  return (bits);   // Return the flags code
}

//=============================================================================================

////////////////////////////////////////////////
// Style Editing Window Code                  //
////////////////////////////////////////////////

// Load the default options.

void Headings_load_defaults()
{
  Headings_Style1 = -1;
  Headings_Style2 = -1;
  Headings_Style3 = -1;
  Headings_Style4 = -1;
  Headings_temp1 = -1;
  Headings_temp2 = -1;
  Headings_temp3 = -1;
  Headings_temp4 = -1;
  Headings_Style1s = "";
  Headings_Style2s = "";
  Headings_Style3s = "";
  Headings_Style4s = "";
  Headings_Alternate = 1;
  Headings_high_water = 1;
  Headings_low_water = 1;
  Headings_RemoveDot = 1;
  Headings_RemoveNum = 1;
  Headings_Separator = "\\t";
}

//---------------------------------------------------------------------------------------------

// Save the current options with the document.

void Headings_save_options()
{
  tagwrites (Headings_current_file, Headings_tag+0, Headings_Style1s);
  tagwrites (Headings_current_file, Headings_tag+1, Headings_Style2s);
  tagwrites (Headings_current_file, Headings_tag+2, Headings_Style3s);
  tagwrites (Headings_current_file, Headings_tag+3, Headings_Style4s);
  tagwrite (Headings_current_file, Headings_tag+4, Headings_Alternate, 0);
  tagwrite (Headings_current_file, Headings_tag+5, Headings_high_water, 0);
  tagwrite (Headings_current_file, Headings_tag+6, Headings_low_water, 0);
  tagwrite (Headings_current_file, Headings_tag+7, Headings_RemoveDot, 0);
  tagwrite (Headings_current_file, Headings_tag+8, Headings_RemoveNum, 0);
  tagwrites (Headings_current_file, Headings_tag+9, Headings_Separator);
}

//---------------------------------------------------------------------------------------------

// Loads the saved options from the document. Returns true always.

int Headings_load_document()
{

  Headings_load_defaults();			// Set default values initially

  if (tagexists(Headings_current_file, Headings_tag+0, 0))
    tagreads (Headings_current_file, Headings_tag+0, Headings_Style1s);
  if (tagexists(Headings_current_file, Headings_tag+1, 0))
    tagreads (Headings_current_file, Headings_tag+1, Headings_Style2s);
  if (tagexists(Headings_current_file, Headings_tag+2, 0))
    tagreads (Headings_current_file, Headings_tag+2, Headings_Style3s);
  if (tagexists(Headings_current_file, Headings_tag+3, 0))
    tagreads (Headings_current_file, Headings_tag+3, Headings_Style4s);
  if (tagexists(Headings_current_file, Headings_tag+4, 0))
    Headings_Alternate = tagread (Headings_current_file, Headings_tag+4, 0);
  if (tagexists(Headings_current_file, Headings_tag+5, 0))
    Headings_high_water = tagread (Headings_current_file, Headings_tag+5, 0);
  if (tagexists(Headings_current_file, Headings_tag+6, 0))
    Headings_low_water = tagread (Headings_current_file, Headings_tag+6, 0);
  if (tagexists(Headings_current_file, Headings_tag+7, 0))
    Headings_RemoveDot = tagread (Headings_current_file, Headings_tag+7, 0);
  if (tagexists(Headings_current_file, Headings_tag+8, 0))
    Headings_RemoveNum = tagread (Headings_current_file, Headings_tag+8, 0);
  if (tagexists(Headings_current_file, Headings_tag+9, 0))
    tagreads (Headings_current_file, Headings_tag+9, Headings_Separator);
  return 1;
}

//---------------------------------------------------------------------------------------------

// Sets the current options from the dialogue box.

void Headings_set_options (int winhandle)
{
  Headings_Style1 = Headings_temp1;          // Permanently set the values for the styles
  Headings_Style2 = Headings_temp2;
  Headings_Style3 = Headings_temp3;
  Headings_Style4 = Headings_temp4;
  if (Headings_Style1 >= 0)
    getuserstyle (Headings_Style1, Headings_Style1s, 0);
  else
    Headings_Style1s = "";
  if (Headings_Style2 >= 0)
    getuserstyle (Headings_Style2, Headings_Style2s, 0);
  else
    Headings_Style2s = "";
  if (Headings_Style3 >= 0)
    getuserstyle (Headings_Style3, Headings_Style3s, 0);
  else
    Headings_Style3s = "";
  if (Headings_Style4 >= 0)
    getuserstyle (Headings_Style4, Headings_Style4s, 0);
  else
    Headings_Style4s = "";
  Headings_Alternate = Headings_AlternateTemp;
  Headings_RemoveDot = Headings_RemoveDotTemp;
  Headings_RemoveNum = Headings_RemoveNumTemp;
  Headings_Separator = readicon(winhandle, 26);

  Headings_parse_separator(Headings_ActualSep);
}

//---------------------------------------------------------------------------------------------

// Display the headings window.

int Headings_StyleWin()
{
  writeicon (Headings_win, 102, Headings_Style1s);
  writeicon (Headings_win, 103, Headings_Style2s);
  writeicon (Headings_win, 104, Headings_Style3s);
  writeicon (Headings_win, 105, Headings_Style4s);
  Headings_Style1 = Headings_findstyle(Headings_Style1s);
  Headings_Style2 = Headings_findstyle(Headings_Style2s);
  Headings_Style3 = Headings_findstyle(Headings_Style3s);
  Headings_Style4 = Headings_findstyle(Headings_Style4s);
  Headings_temp1 = Headings_Style1;
  Headings_temp2 = Headings_Style2;
  Headings_temp3 = Headings_Style3;
  Headings_temp4 = Headings_Style4;
  Headings_AlternateTemp = Headings_Alternate;
  select_icon (Headings_win, 18, Headings_Alternate);
  Headings_RemoveDotTemp = Headings_RemoveDot;
  select_icon (Headings_win, 24, Headings_RemoveDot);
  Headings_RemoveNumTemp = Headings_RemoveNum;
  select_icon (Headings_win, 25, Headings_RemoveNum);
  writeicon (Headings_win, 26, Headings_Separator);

  if (Headings_RemoveNum)
    set_icon_state(Headings_win, 26, 0x00000000, 0x00400000);
  else
    set_icon_state(Headings_win, 26, 0x00400000, 0x00400000);

  Headings_check_ready();

  display_window(Headings_win, 20, 1);

  return(1);
}

//---------------------------------------------------------------------------------------------

// Opens the menus off the buttons on the headings style window.

void Headings_openwinmenu (int winhandle, int icon)
{
  Headings_current_icon = icon;           // set current icon
  Headings_choosestyle();        // make the menu
  open_menu(Headings_stylemenu); // show the menu
}

//---------------------------------------------------------------------------------------------




void Headings_close(int handle)
{
 if(Headings_win)
 {
  close_window(Headings_win);
  delete_window(Headings_win);
  Headings_win=0;
  Headings_current_file=0;
  Headings_current_view=0;
 }
}





//  Mouse click event handler for the headings style window

void Headings_click (int winhandle, int icon, int bbits, int mx, int my)
{
  int tempbm;

  if (bbits != 4 && bbits != 1) return;	// Return if not left or right mouse button

  switch(icon)
  {
    case 1:                    // Default action window, produce the headings

      if (Headings_check_ready())
      {
        Headings_set_options(winhandle);
        tempbm = bmcreate("Headings_SFind");       // Used when searching through the text
        bmstartscan();
        bmnextstory(tempbm);
        bmmove(tempbm, 0, 4);			 // Make sure we are at the start of the text
        setcarettobm(tempbm);			 // Start at beginning of document
        Headings_do_headings(tempbm);
        Headings_save_options();
        Headings_current_bm = 0;

        if(bbits==4) Headings_close(winhandle);
      }
      break;

    case 2:                   // Cancel button  - closes window
      Headings_current_bm = 0;
      Headings_close(winhandle);
      break;

    case 19:                 // Set values
      Headings_set_options(winhandle);
      Headings_save_options();
      Headings_current_bm = 0;
      if(bbits==4) Headings_close(winhandle);
      break;

    case 22:			// Set a Heading Location
      if (caretchar() >= 0)
      {
        if (!Headings_find_bookmark())
        {
          // Create a new headings bookmark and place it at the caret.

          tempbm = bmcreate("HeaD" + itos(Headings_high_water));
          Headings_high_water++;
          setbmtocaret(tempbm);
          Headings_check_ready();
          Headings_save_options();
        }
        else
          bbc_vdu(7);
      }
      else
        bbc_vdu(7);
      break;

    case 23:			// Remove a headings bookmark
      if (caretchar() >= 0)
      {
        if ((tempbm = Headings_find_bookmark()))
        {
          bmdelete(tempbm);
          Headings_save_options();
        }
        else
          bbc_vdu(7);
      }
      else
        bbc_vdu(7);
      break;

    case 24:		       // Remove trailing period.
      select_icon (winhandle, 24, Headings_RemoveDotTemp = !Headings_RemoveDotTemp);
      break;

    case 25:		       // Remove section numbers
      select_icon (winhandle, 25, Headings_RemoveNumTemp = !Headings_RemoveNumTemp);
      if (Headings_RemoveNumTemp)
        set_icon_state(Headings_win, 26, 0x00000000, 0x00400000);
      else
        set_icon_state(Headings_win, 26, 0x00400000, 0x00400000);
      break;

    case 28:                   // Find the next heading location
      Headings_next_location();
      break;

    case 6:                    // All the menu buttons
    case 7:
    case 8:
    case 9:
      // Open the menu off the menu buttons.
      Headings_openwinmenu (winhandle, icon);
      break;

    case 18:			// Alternate headings options.
      select_icon (winhandle, 18, Headings_AlternateTemp = !Headings_AlternateTemp);
      break;

    default:                   // click anywhere else - don't do anything
      break;

  }
}

//---------------------------------------------------------------------------------------------

// Key press event handler for headings style window.

int Headings_key_press (int winhandle, int icon, int key)
{
  switch (key)
  {
    case 13:			// Return, simulate mouse click over Do Headings
      Headings_click (winhandle, 1, 4, 0, 0);
      break;

    case 27:			// Esc, simulate mouse click over Cancel
      Headings_click (winhandle, 2, 4, 0, 0);
      break;

    default:
      break;
   }

   return key;
}

//=============================================================================================







////////////////////////////////////////////////
// Headings Menu Functions                    //
////////////////////////////////////////////////

// Called when the Headings menu entry is clicked.

int Headings_command (int n, int sub)
{
 Headings_close(Headings_win);

 // Check if the file has changed.

 if(Headings_current_file!=currentfile())
 {
  Headings_current_file=currentfile();	// Get file handle
  Headings_load_document();			// Load options saved with document
 }

 Headings_current_view=currentview();

 Headings_win=create_windowparent("Headings",viewhandle(currentview()));

 // Add handlers for when the user clicks on the headings styles window or when
 // s(he) hits a key with it displayed.

 addwindowhandler(0, Headings_win, "Headings_click");
 addwindowhandler(1, Headings_win, "Headings_key_press");
 addwindowhandler(7, Headings_win, "Headings_close");
 addwindowhandler(2, Headings_win, "Headings_close");


 Headings_StyleWin();				// Display the Headings window

 setactive(Headings_win);
}

//=============================================================================================

////////////////////////////////////////////////
// Event handlers.			      //
////////////////////////////////////////////////

// Called when a file is closed.

void Headings_file_closing (int user, int file)
{
 if(file==Headings_current_file)
 {
  Headings_close(Headings_win);
 }
}


//---------------------------------------------------------------------------------------------

// Called when a view is made active.

void Headings_view_active (int user, int view, int gain)
{
 if(view==Headings_current_view)
 {
  if(!gain)
  {
   Headings_close(Headings_win);
  }
 }
}

//---------------------------------------------------------------------------------------------

// Called when a view is closed.

void Headings_view_closed (int user, int view)
{
 if(view==Headings_current_view)
 {
  Headings_close(Headings_win);
 }
}

//=============================================================================================

////////////////////////////////////////////////
// Main Functions                             //
////////////////////////////////////////////////

void main(void)
{
  string help = "{HEADINGS_10}";

  Headings_load_defaults ();	// Set the default options

  // Create the menus and windows, assign to handles.

  Headings_stylemenu = create_menu("{HEADINGS_05}");


  // Add headings menu to applets menu.

  script_menu_initialise();             // Initialise the menu
  script_menu_insert("Headings_command", "", "", "", "{HEADINGS_00}");


  // Add handlers for files and views being closed and views made active.

  addeventhandler (0x02, 0, "Headings_file_closing");
  addeventhandler (0x06, 0, "Headings_view_closed");
  addeventhandler (0x215,0, "Headings_view_active");

  // Define a macro to launch us off.

  if(isdefmacro("headings")==-1)
  {
   translate(help);
   defmacro3(0, "headings", "{Headings_command(0,0)}", "headings", help, "Headings");
  }


  bm_hide("HeaD");
}



