// !Sections v1.11

// Inserts section numbers by scanning through all the text and looking
// for headings of particular styles and formatting appropriately.
// Written by Roger Noble, March 2000

// Version History:
// v1.00 (16th March 2000):  Initial version.
// v1.01 (1st June 2000):    Added missing setting of period option when window first opened.
// v1.02 (6th April 2001):   Display hour glass when busy.
// v1.10 (11th June 2002):   Added option for doing continuous level 1 numbering over chapters.
// v1.11 (13th June 2002):   Do sections from cursor with all chapters selected now just does
//                           chapters from the current position up to the last.

//=============================================================================================

// Global Variables.

// Options.

int Sections_PrefixOpt = 0;     // Print a prefix text before section numbers, 1 = yes
int Sections_AutoOpt = 0;       // Base section numbers on chapter number, 1 = yes
int Sections_AllChapsOpt = 0;   // Do all chapters, 1 = yes
int Sections_PeriodOpt = 0;     // Put a full stop following level 1 headings, 1 = yes
int Sections_NumberFormat = 2;  // Number format: 0 = Roman upper, 1 = Roman lower,
                                // 2 = Arabic
int Sections_ContNumOpt = 0;	// Continuous numbering over chapters

string Sections_Prefix;         // The string printed before section numbers
string Sections_Start;          // The starting number as a string
string Sections_Separator;      // The separator to use between section numbers and the heading

int Sections_PrefixOptTemp;	// Used to hold values whilst Sections dialogue active
int Sections_AutoOptTemp;
int Sections_AllChapsOptTemp;
int Sections_PeriodOptTemp;
int Sections_NumberFormatTemp;
int Sections_ContNumOptTemp;

// Handles and techie things.

int Sections_stylemenu;     // Style menu-button menu handle
int Sections_win;           // The Edit Style Window handle

// Styles.

int Sections_Style1 = -1;   // Style handles
int Sections_Style2 = -1;
int Sections_Style3 = -1;
int Sections_Style4 = -1;
string Sections_Style1s;    // Style names
string Sections_Style2s;
string Sections_Style3s;
string Sections_Style4s;
int Sections_temp1 = -1;    // Used in Sections dialogue to hold style handles
int Sections_temp2 = -1;
int Sections_temp3 = -1;
int Sections_temp4 = -1;

// Starting section numbers, one for each level.

int Sections_Start1 = -1;
int Sections_Start2 = -1;
int Sections_Start3 = -1;
int Sections_Start4 = -1;

// Current highest level section number.

int Sections_Current1;

string Sections_ParsedSep;   // Parsed seperator for comparison with text in document
string Sections_InsertSep;   // Parsed separator for inserting after numbers

// Window icons.

int Sections_current_icon = 0; // Which style am I currently editing
                               // Bad style to have it global, but this is the simplest
                               // way of getting the job done

// Document-related stuff.

int Sections_current_file = 0; 			       // Handle to current file
int Sections_current_view = 0;			       // Handle to current view

// Constants.

int Sections_tag = ('s'<<24) | ('e'<<16) | ('c'<<8);   // Base tag value
int Sections_truncate = 512;			       // (****) Truncate a heading if longer than this

//=============================================================================================

////////////////////////////////////////////////
// Sections Page Producing Functions          //
////////////////////////////////////////////////

// Test if the style is one of the required styles. Returns the
// style number, or zero if not.

int Sections_isstyle()
{
  caretcontext();

  iscurrentstyle(0);			      // Set up styles

  if (iscurrentstyle(Sections_Style1) != 0)   // Is it style 1 at the caret?
    return(1);                                // If so return 1
  if (iscurrentstyle(Sections_Style2) != 0)   // and so on
    return(2);
  if (iscurrentstyle(Sections_Style3) != 0)
    return(3);
  if (iscurrentstyle(Sections_Style4) != 0)
    return(4);

  return(0);                       // It wasn't any of the known styles
}

//---------------------------------------------------------------------------------------------

// Find a style with the name "stylename", return the handle for it. Returns -1
// if the style is not found.

int Sections_findstyle(string stylename)
{
  int temp;
  string tempstring;

  for (temp = 0; getuserstyle(temp, tempstring, 0) != 0; temp++)
  {
    if (tempstring == stylename)
      return(temp);
  }

  return -1;
}

//---------------------------------------------------------------------------------------------

// Sets the style handles from the style names.

void Sections_set_styles()
{
    Sections_Style1 = Sections_findstyle (Sections_Style1s);
    Sections_Style2 = Sections_findstyle (Sections_Style2s);
    Sections_Style3 = Sections_findstyle (Sections_Style3s);
    Sections_Style4 = Sections_findstyle (Sections_Style4s);
    Sections_temp1 = Sections_Style1;
    Sections_temp2 = Sections_Style2;
    Sections_temp3 = Sections_Style3;
    Sections_temp4 = Sections_Style4;
}

//---------------------------------------------------------------------------------------------

// Does the section numbers for one story. Returns zero if there are more
// chapters (stories) to do. tempbm should be a bookmark in the required
// story.

int Sections_styletest (int tempbm, int remove)
{
  int level;
  int previous_level = 0;
  int i = 1;
  int j;
  int to_delete;
  int end;
  int bm;
  int bm1;
  int bm2;
  int bm3;
  int c;
  int chapter;
  int numberformat;
  int secnum1, secnum2, secnum3, secnum4;
  string start;
  int prefix_len;
  int start_len;
  int style;
  int separator_len = slen(Sections_ParsedSep);
  int separator_count;

  bm = bmcreate ("Sections_SBM");        // Bookmark for going through current story
  setbmtobm(bm, tempbm);
  bm1 = bmcreate ("Sections_SBM1");	 // Bookmark for going through story
  bm2 = bmcreate ("Sections_SBM2");	 // Bookmark for checking
  bm3 = bmcreate ("Sections_SBM3");      // Bookmark for use when checking for a separator

  if (Sections_AutoOpt && !remove)
  {
    chapter = bmchapter(tempbm) + chapterstart(currentfile());

    // Obtain the chapter number format. This is not supported for versions of OvP prior
    // to 2.57. We could test for the version number here, but since the earlier versions
    // don't have itors() the applet will not load properly with these anyway.

    numberformat = (chapterbits(bmchapter(tempbm), currentfile()) >> 4) & 0xF;

    secnum1 = chapter;
    secnum2 = 1;
    secnum3 = 1;
    secnum4 = 1;
  }
  else
  {
    secnum1 = Sections_Current1;
    secnum2 = Sections_Start2;
    secnum3 = Sections_Start3;
    secnum4 = Sections_Start4;

    numberformat = Sections_NumberFormat;
  }

  if (Sections_PrefixOpt)
    prefix_len = slen(Sections_Prefix);
  else
    prefix_len = 0;

  while (i != 0)                         // While not the end of text
  {
    setcarettobm(bm);                    // Move the caret to the bookmark
    setbmtobm(bm1,bm);			 // Bookmark at start of selected text
    i = bmmove(bm, 1, 3);             	 // Move bm to end of paragraph
    setzonetobm(bm);                 	 // Select the bit between caret & bm
    level = Sections_isstyle();          // Find out what style it is, and is it one we
                                         // are interested in

    if (candelete() != 0 && level != 0)  // If I can in principle delete this selection and
                                         // its a style I need
    {
      // Got something to process. First we must remove any prefix text and following
      // numbers. The algorithm that we use is as follows:
      //  1. If the text starts with the prefix then we remove that and any following
      //     number upto but not including a space or tab.
      //  2. If the text does not start with the prefix then we remove any starting
      //     number.
      //  3. Otherwise we remove nothing, but if the text does not start with the separator
      //     we insert one.
      //  "number" here is defined as any number of arabic numerals and "." characters,
      //  or any mixture of ivxlcdm (upper or lower case) and "." characters according to
      // the current number format.

      // Go through the heading copying it into a local string until we reach the
      // separator, or the end of the heading.

      start = "";
      separator_count = 0;
      do
      {
        c = bmchar(bm1);               // Current character

        if (c == schar(Sections_ParsedSep, separator_count) && slen(start) > prefix_len)
        {
          // Possible separator.

          if (separator_count++ == 0) setbmtobm(bm3,bm1);   // Remember start of potential separator

          // Check for end of separator.

          if (separator_count >= separator_len) setbmtobm(bm1,bm); // End of separator, force a finish
        }
        else if (separator_count > 0)
        {
          // Not a separator, go back to where we were and add the characters to the string

          setbmtobm(bm1, bm3);
          c = bmchar(bm1);
          separator_count = 0;
        }

        if (separator_count == 0)            // Add character to string if not in potential separator
          start += chars(c);

	setbmtobm (bm2, bm1);			// Remember where we are
        end = bmmove(bm1, 1, 0);         	// Next character
	if (bmcmp(bm1,bm2) <= 0) end = 0;	// If we did not move, finish
      }
      while (end != 0 && bmcmp(bm1,bm) < 0)

      // Did we reach the end of the heading whilst searching for a separator?

      if (separator_count > 0 && separator_count < separator_len)
      {
        // We did, copy the rest of the stuff to the starting string.

        setbmtobm(bm1, bm3);
        for (i = separator_count; i < separator_count; ++i)
        {
          c = bmchar(bm1);
          start += chars(c);
          bmmove(bm1, 1, 0);
        }
      }

      // start now contains the starting text of the heading. Check it for the
      // prefix.

      start_len = slen(start);

      if (Sections_PrefixOpt && prefix_len <= start_len && Sections_Prefix == mids(start, 0, prefix_len))
      {
        // We have got a prefix.
        // Keep going until we have found the end of any following number. Numbers
        // are as defined above.

        if (separator_count > 0)
        {
          start = mids(start, prefix_len, start_len-prefix_len);

          // Obtain the longest string containing the write format digits.

          if (Sections_NumberFormat == 2)
            to_delete = start % "0123456789.";
          else if (Sections_NumberFormat == 1)
            to_delete = start % "ivxlcdm.";
          else
            to_delete = start % "IVXLDCM.";

          if (slen(start) - to_delete >= separator_len)
          {
            if (mids(start, to_delete, separator_len) != Sections_ParsedSep)
              to_delete = 0;   // Number not followed by separator
          }
          else if (to_delete != slen(start))
            to_delete = 0;     // Number not followed by separator
        }
        else
          to_delete = 0;       // No separator

        to_delete += prefix_len;                  // Number of characters to delete
      }
      else
      {
        // There is no prefix, just remove any starting number.

        if (separator_count > 0)
        {
          // Obtain the longest string containing the write format digits.

          if (Sections_NumberFormat == 2)
            to_delete = start % "0123456789.";
          else if (Sections_NumberFormat == 1)
            to_delete = start % "ivxlcdm.";
          else
            to_delete = start % "IVXLDCM.";

          if (slen(start) - to_delete >= separator_len)
          {
            if (mids(start, to_delete, separator_len) != Sections_ParsedSep)
            {
              to_delete = 0;           // No separator following number
            }
          }
          else if (to_delete != slen(start))
            to_delete = 0;             //  No separator following number
        }
        else
          to_delete = 0;     // No separator found
      }

      clearselection();

      // If we are removing the numbers then remove the separator as well.

      if (remove) to_delete += separator_len;

      // Remove the prefix and any following number (if required). The caret should still
      // be at the start of the heading.

      if (to_delete > 0)
      {
        setbmtocaret (bm2);
        for (j = 0; j < to_delete; ++j)
          bmmove (bm2, 1, 0);
        setzonetobm (bm2);
        deleteselection();
      }

      // Insert the prefix and number if not just removing them.

      if (!remove)
      {
        // Construct a section number and enter it into the document. The format is
        // decided by the heading level that we are dealing with.

        if (level < previous_level)
        {
          switch (level)
          {
            case 1:
              secnum2 = 1;
            case 2:
              secnum3 = 1;
            case 3:
              secnum4 = 1;
          }
        }
        previous_level = level;

        if (numberformat >= 2)    // Arabic
          switch (level)
          {
            case 1:
              start = itos(secnum1++);
              break;
            case 2:
              start = itos(secnum1-1) + "." + itos(secnum2++);
              break;
            case 3:
              start = itos(secnum1-1) + "." + itos(secnum2-1) + "." + itos(secnum3++);
              break;
            case 4:
              start = itos(secnum1-1) + "." + itos(secnum2-1) + "." + itos(secnum3-1)
                      + "." + itos(secnum4++);
              break;
            default:
              start = "";
          }
        else                  // Roman
        {
          switch (level)
          {
            case 1:
              start = itors(secnum1++);
              break;
            case 2:
              start = itors(secnum1-1) + "." + itors(secnum2++);
              break;
            case 3:
              start = itors(secnum1-1) + "." + itors(secnum2-1) + "." + itors(secnum3++);
              break;
            case 4:
              start = itors(secnum1-1) + "." + itors(secnum2-1) + "." + itors(secnum3-1)
                      + "." + itors(secnum4++);
              break;
            default:
              start = "";
          }

          if (numberformat == 1) start = -start;         // Lower case Roman
        }

        style = Sections_findstyle("Sections" + itos(level));
        if (style >= 0) addstyle(style);
        if (Sections_PrefixOpt) type(Sections_Prefix);
        type(start);
        if (Sections_PeriodOpt && level == 1) type(".");     // Do following period if requested
        if (to_delete <= 0 && separator_count < separator_len) type(Sections_InsertSep); // Insert separator if necessary
        if (style >= 0) remstyle(style);
      }
    }

    clearselection();	// To be tidy

    // Next paragraph.

    setbmtobm (bm2, bm);			// Remember where we are
    i = bmmove(bm, 1, 7);       		// Move down a paragraph
    if (i != 0 && bmcmp(bm,bm2) <= 0) i = 0;	// If we did not move, finish story
  }

  bmdelete (bm3);
  bmdelete (bm2);
  bmdelete (bm1);
  bmdelete (bm);

  if (Sections_ContNumOpt) Sections_Current1 = secnum1;  // Continue with the numbering for the next chapter if requested

  chapter = bmchapter(tempbm);           // Current chapter
  i = bmnextstory(tempbm);		 // Move to next story
  if (!i && chapter == bmchapter(tempbm))
  {
    // Still in the current chapter, try to get the next one.

    i = bmnextstory(tempbm);
  }

  return i;
}

//---------------------------------------------------------------------------------------------

// Produce the section numbers.

void Sections_do_sections (int tempbm, int remove)
{
  int chapter;

//  swi13(0x406c0, 0, 0, 0);

  Sections_set_styles();

  Sections_Current1 = Sections_Start1;		  // Starting highest level section number

  if (Sections_AllChapsOpt)
  {
    // Do all chapters.

    while (!Sections_styletest(tempbm, remove));
  }
  else
  {
    // Just do the current chapter.

    Sections_styletest(tempbm, remove);           // Just the current chapter
  }

  bmdelete (tempbm);

//  swi13(0x406c1, 0, 0, 0);
}

//=============================================================================================

////////////////////////////////////////////////
//  Sections Style Menu Functions             //
////////////////////////////////////////////////

// Checks if any sections styles have been set.
// Returns 1 if any styles have been set, 0 otherwise.

int Sections_check_ready()
{
  if (Sections_temp1 <= -1 && Sections_temp2 <= -1 && Sections_temp3 <= -1
      && Sections_temp4 <= -1)
  {
    // No sectionss styles are set. Disable the Do Sections actions.

    set_icon_state (Sections_win, 30, 0x00400000, 0x00400000);
    set_icon_state (Sections_win, 21, 0x00400000, 0x00400000);
    set_icon_state (Sections_win, 0, 0x00400000, 0x00400000);

    return 0;
  }
  else
  {
    // Allow all actions.

    set_icon_state (Sections_win, 30, 0x00000000, 0x00400000);
    set_icon_state (Sections_win, 21, 0x00000000, 0x00400000);
    set_icon_state (Sections_win, 0, 0x00000000, 0x00400000);

    return 1;
  }
}

//---------------------------------------------------------------------------------------------

int Sections_choosestyle(void)
{
  int i = 0;
  string tempstring = "";

  if (Sections_stylemenu)
  {
    delete_menu(Sections_stylemenu);
    Sections_stylemenu = create_menu("{SECTIONS_05}");
  }

  // Add all the menu entries.

  addentry_menu (Sections_stylemenu, "Sections_styleentry", "Sections_styleflags", "", "", "Clear Style");

  for (i = 0; getuserstyle(i, tempstring, 0) != 0; i++)
  {
    addentry_menu(Sections_stylemenu, "Sections_styleentry", "Sections_styleflags", "", "", tempstring);
  }
}

//---------------------------------------------------------------------------------------------

int Sections_styleentry (int n, int subcode)
{
  string stylename;

  if (n > 0)
    getuserstyle (n-1, stylename, 0);
  else
    stylename = "";		// Clear style
  writeicon (Sections_win, (Sections_current_icon-4+100), stylename);

  switch (Sections_current_icon)
  {
    case 6:
      Sections_temp1 = n-1;
      break;

    case 7:
      Sections_temp2 = n-1;
      break;

    case 8:
      Sections_temp3 = n-1;
      break;

    case 9:
      Sections_temp4 = n-1;
      break;

    default:
      return (0);
      break;
  }

  Sections_check_ready();	// Check if ready to do the section numbers now

  return (1);
}

//---------------------------------------------------------------------------------------------

// Tick, shade etc the items appropriately.

int Sections_styleflags (int n, string &s)
{
  int bits = 0;

  switch (Sections_current_icon)   // Find out which icon we are editing
  {
    case 6:
      if(Sections_temp1 == n-1)    // is the current style = this menu item
        bits = TICKED;     // if so then tick it
      break;

    case 7:
      if(Sections_temp2 == n-1)    // and so on...
        bits = TICKED;
      break;

    case 8:
      if(Sections_temp3 == n-1)
        bits = TICKED;
      break;

    case 9:
      if(Sections_temp4 == n-1)
        bits = TICKED;
      break;

    default:
      return (0);
      break;
  }

  return (bits);   // Return the flags code
}

//=============================================================================================

////////////////////////////////////////////////
// Style Editing Window Code                  //
////////////////////////////////////////////////

// Load the default options.

void Sections_load_defaults()
{
  Sections_Style1 = -1;
  Sections_Style2 = -1;
  Sections_Style3 = -1;
  Sections_Style4 = -1;
  Sections_temp1 = -1;
  Sections_temp2 = -1;
  Sections_temp3 = -1;
  Sections_temp4 = -1;
  Sections_Style1s = "";
  Sections_Style2s = "";
  Sections_Style3s = "";
  Sections_Style4s = "";
  Sections_AutoOpt = 0;
  Sections_PrefixOpt = 0;
  Sections_PeriodOpt = 0;
  Sections_AllChapsOpt = 0;
  Sections_NumberFormat = 2;       // Arabic
  Sections_ContNumOpt = 0;
  Sections_Prefix = "";
  Sections_Start = "1";
  Sections_Separator = "\\t";       // tab
}

//---------------------------------------------------------------------------------------------

// Save the current options with the document.

void Sections_save_options()
{
  tagwrites (Sections_current_file, Sections_tag+0, Sections_Style1s);
  tagwrites (Sections_current_file, Sections_tag+1, Sections_Style2s);
  tagwrites (Sections_current_file, Sections_tag+2, Sections_Style3s);
  tagwrites (Sections_current_file, Sections_tag+3, Sections_Style4s);
  tagwrite (Sections_current_file, Sections_tag+4, Sections_AutoOpt, 0);
  tagwrite (Sections_current_file, Sections_tag+5, Sections_PrefixOpt, 0);
  tagwrite (Sections_current_file, Sections_tag+6, Sections_AllChapsOpt, 0);
  tagwrite (Sections_current_file, Sections_tag+7, Sections_NumberFormat, 0);
  tagwrites (Sections_current_file, Sections_tag+8, Sections_Prefix);
  tagwrites (Sections_current_file, Sections_tag+9, Sections_Start);
  tagwrites (Sections_current_file, Sections_tag+10, Sections_Separator);
  tagwrite (Sections_current_file, Sections_tag+11, Sections_PeriodOpt, 0);
  tagwrite (Sections_current_file, Sections_tag+12, Sections_ContNumOpt, 0);
}

//---------------------------------------------------------------------------------------------

// Loads the saved options from the document. Returns true always.

int Sections_load_document()
{

  Sections_load_defaults();			// Set default values initially

  if (tagexists(Sections_current_file, Sections_tag+0, 0))
    tagreads (Sections_current_file, Sections_tag+0, Sections_Style1s);
  if (tagexists(Sections_current_file, Sections_tag+1, 0))
    tagreads (Sections_current_file, Sections_tag+1, Sections_Style2s);
  if (tagexists(Sections_current_file, Sections_tag+2, 0))
    tagreads (Sections_current_file, Sections_tag+2, Sections_Style3s);
  if (tagexists(Sections_current_file, Sections_tag+3, 0))
    tagreads (Sections_current_file, Sections_tag+3, Sections_Style4s);
  if (tagexists(Sections_current_file, Sections_tag+4, 0))
    Sections_AutoOpt = tagread (Sections_current_file, Sections_tag+4, 0);
  if (tagexists(Sections_current_file, Sections_tag+5, 0))
    Sections_PrefixOpt = tagread (Sections_current_file, Sections_tag+5, 0);
  if (tagexists(Sections_current_file, Sections_tag+6, 0))
    Sections_AllChapsOpt = tagread (Sections_current_file, Sections_tag+6, 0);
  if (tagexists(Sections_current_file, Sections_tag+7, 0))
    Sections_NumberFormat = tagread (Sections_current_file, Sections_tag+7, 0);
  if (tagexists(Sections_current_file, Sections_tag+8, 0))
    tagreads (Sections_current_file, Sections_tag+8, Sections_Prefix);
  if (tagexists(Sections_current_file, Sections_tag+9, 0))
    tagreads (Sections_current_file, Sections_tag+9, Sections_Start);
  if (tagexists(Sections_current_file, Sections_tag+10, 0))
    tagreads (Sections_current_file, Sections_tag+10, Sections_Separator);
  if (tagexists(Sections_current_file, Sections_tag+11, 0))
    Sections_PeriodOpt = tagread (Sections_current_file, Sections_tag+11, 0);
  if (tagexists(Sections_current_file, Sections_tag+12, 0))
    Sections_ContNumOpt = tagread (Sections_current_file, Sections_tag+12, 0);
  return 1;
}

//---------------------------------------------------------------------------------------------

// Parses the starting number string and sets the starting numbers.

void Sections_set_start()
{
  int len;
  int number;
  int i = 0;
  int j;
  int level = 1;
  Sections_Start1 = 1;   // Default starting numbers are 1
  Sections_Start2 = 1;
  Sections_Start3 = 1;
  Sections_Start4 = 1;

  // Go through the string setting the starting numbers.

  len = slen(Sections_Start);

  while (i < len)
  {
    if (mids(Sections_Start, i, 1) == ".")
    {
      if (level >= 4) break;
      ++level;
      ++i;
    }
    else
    {
      j = mids(Sections_Start, i, len-i) % "0123456789";
      if (j <= 0) return;      // Not a digit
      number = stoi(mids(Sections_Start, i, j));
      switch (level)
      {
        case 1:
          Sections_Start1 = number;
          break;
        case 2:
          Sections_Start2 = number;
          break;
        case 3:
          Sections_Start3 = number;
          break;
        case 4:
          Sections_Start4 = number;
          break;
      }

      i += j;
    }
  }

  return;
}

//---------------------------------------------------------------------------------------------

// Parses the separator string provided and returns a string suitable for direct
// comparison with the in-line text.

void Sections_parse_separator(string& separator, string& insert)
{
  int separator_len = slen(Sections_Separator);
  int i = 0;
  int c;

  separator = "";
  insert = "";

  while (i < separator_len)
  {
    c = schar(Sections_Separator, i);

    if (c == '\\')
    {
      // Quoted character, check the next one.

      if (i+1 < separator_len)
      {
        c = schar(Sections_Separator, i+1);

        switch (c)
        {
          case 't':    // Tab
            separator += chars(18);
            insert += "{tab}";
            break;

          case 'r':   // Newlines and the like, replace with space
          case 'n':
            separator += " ";
            insert += " ";
            break;

          default:    // Others, replace with the character following the '\'
            separator += chars(c);
            insert += chars(c);
            break;
        }

        i += 2;
      }
    }
    else if (c >= 32)
    {
      separator += chars(c);   // Normal character
      insert += chars(c);
      ++i;
    }
    else
      ++i;                     // Ignore control characters
  }
}

//---------------------------------------------------------------------------------------------

// Sets the current options from the dialogue box.

void Sections_set_options (int winhandle)
{
  Sections_Style1 = Sections_temp1;          // Permanently set the values for the styles
  Sections_Style2 = Sections_temp2;
  Sections_Style3 = Sections_temp3;
  Sections_Style4 = Sections_temp4;
  if (Sections_Style1 >= 0)
    getuserstyle (Sections_Style1, Sections_Style1s, 0);
  else
    Sections_Style1s = "";
  if (Sections_Style2 >= 0)
    getuserstyle (Sections_Style2, Sections_Style2s, 0);
  else
    Sections_Style2s = "";
  if (Sections_Style3 >= 0)
    getuserstyle (Sections_Style3, Sections_Style3s, 0);
  else
    Sections_Style3s = "";
  if (Sections_Style4 >= 0)
    getuserstyle (Sections_Style4, Sections_Style4s, 0);
  else
    Sections_Style4s = "";
  Sections_AutoOpt = Sections_AutoOptTemp;
  Sections_PrefixOpt = Sections_PrefixOptTemp;
  Sections_AutoOpt = Sections_AutoOptTemp;
  Sections_PeriodOpt = Sections_PeriodOptTemp;
  Sections_AllChapsOpt = Sections_AllChapsOptTemp;
  Sections_Prefix = readicon(winhandle, 20);
  Sections_Start = readicon(winhandle, 28);
  Sections_Separator = readicon(winhandle, 32);
  Sections_NumberFormat = Sections_NumberFormatTemp;
  Sections_ContNumOpt = Sections_ContNumOptTemp;

  Sections_set_start();                         // Set the starting numbers
  Sections_parse_separator(Sections_ParsedSep, Sections_InsertSep); // Set the separators to use
}

//---------------------------------------------------------------------------------------------

// Display the sections style window.

int Sections_StyleWin()
{
  writeicon (Sections_win, 102, Sections_Style1s);
  writeicon (Sections_win, 103, Sections_Style2s);
  writeicon (Sections_win, 104, Sections_Style3s);
  writeicon (Sections_win, 105, Sections_Style4s);
  Sections_Style1 = Sections_findstyle(Sections_Style1s);
  Sections_Style2 = Sections_findstyle(Sections_Style2s);
  Sections_Style3 = Sections_findstyle(Sections_Style3s);
  Sections_Style4 = Sections_findstyle(Sections_Style4s);
  Sections_temp1 = Sections_Style1;
  Sections_temp2 = Sections_Style2;
  Sections_temp3 = Sections_Style3;
  Sections_temp4 = Sections_Style4;
  Sections_AutoOptTemp = Sections_AutoOpt;
  Sections_PrefixOptTemp = Sections_PrefixOpt;
  Sections_PeriodOptTemp = Sections_PeriodOpt;
  Sections_AllChapsOptTemp = Sections_AllChapsOpt;
  Sections_NumberFormatTemp = Sections_NumberFormat;
  Sections_ContNumOptTemp = Sections_ContNumOpt;
  select_icon (Sections_win, 27, Sections_AutoOpt);
  select_icon (Sections_win, 31, Sections_AllChapsOpt);
  select_icon (Sections_win, 18, Sections_PrefixOpt);
  select_icon (Sections_win, 34, Sections_PeriodOpt);
  select_icon (Sections_win, 35, Sections_ContNumOpt);
  if (Sections_NumberFormat == 2)
  {
    select_icon (Sections_win, 24, 1);
    select_icon (Sections_win, 25, 0);
    select_icon (Sections_win, 26, 0);
  }
  else if (Sections_NumberFormat == 1)
  {
    select_icon (Sections_win, 24, 0);
    select_icon (Sections_win, 25, 1);
    select_icon (Sections_win, 26, 0);
  }
  else
  {
    select_icon (Sections_win, 24, 0);
    select_icon (Sections_win, 25, 0);
    select_icon (Sections_win, 26, 1);
  }

  writeicon (Sections_win, 20, Sections_Prefix);
  writeicon (Sections_win, 28, Sections_Start);
  writeicon (Sections_win, 32, Sections_Separator);

  if (Sections_PrefixOpt)
    set_icon_state (Sections_win, 20, 0x00000000, 0x00400000);
  else
    set_icon_state (Sections_win, 20, 0x00400000, 0x00400000);

  if (Sections_AutoOpt)
    set_icon_state (Sections_win, 28, 0x00400000, 0x00400000);
  else
    set_icon_state (Sections_win, 28, 0x00000000, 0x00400000);

  if (Sections_AllChapsOpt && !Sections_AutoOpt)
    set_icon_state (Sections_win, 35, 0x00000000, 0x00400000);
  else
    set_icon_state (Sections_win, 35, 0x00400000, 0x00400000);

  Sections_check_ready();

  display_window(Sections_win, 20, 1);

  return(1);
}

//---------------------------------------------------------------------------------------------

// Opens the menus off the buttons on the sections style window.

void Sections_openwinmenu (int winhandle, int icon)
{
  Sections_current_icon = icon;           // set current icon
  Sections_choosestyle();        // make the menu
  open_menu(Sections_stylemenu); // show the menu
}

//---------------------------------------------------------------------------------------------



void Sections_close(int handle)
{
 if(Sections_win)
 {
  close_window(Sections_win);
  delete_window(Sections_win);
  Sections_win=0;
  Sections_current_file=0;
  Sections_current_view=0;
 }
}





//  Mouse click event handler for the sections style window

void Sections_click (int winhandle, int icon, int bbits, int mx, int my)
{
  int tempbm;
  int tempbm1;
  int chapter;
  int remove = 0;

  if (bbits != 4 && bbits != 1) return;	// Return if not left or right mouse button

  switch(icon)
  {
    case 30:                   // Remove section numbers
      remove = 1;
      // Fall through to case 0.

    case 1:                    // Default action window, produce the section numbers

      if (Sections_check_ready())
      {
        Sections_set_options(winhandle);
        Sections_save_options();

        // Check that the caret is anywhere.

        if (caretchar() >= 0)
        {

          tempbm = bmcreate("Sections_SFind");     // Used when searching through the text

          bmstartscan();
          if (Sections_AllChapsOpt)
          {
            bmnextstory(tempbm);
            bmmove(tempbm, 0, 4);
            setcarettobm(tempbm);	           // Start at beginning of document
          }
          else
          {
            setbmtocaret(tempbm);
            bmmove(tempbm, 0, 4);                  // Start of beginning of current chapter
          }

          Sections_do_sections(tempbm, remove);
        }
        else
        {
          // There is no known cursor position.

          string s = "{SECTIONS_09}";
          translate(s);
          messagebox(s);
        }

        if (bbits == 4) Sections_close(winhandle);
      }
      break;

    case 2:                   // Cancel button  - closes window
      Sections_close(winhandle);
      break;

    case 21:		       // Do Sections from cursor.
      Sections_set_options(winhandle);
      Sections_save_options();

      // Check if the caret is anywhere.

      if (caretchar() >= 0)
      {
         tempbm = bmcreate("Sections_SFind");
         tempbm1 = bmcreate("Sections_SCurrent");
         bmstartscan();
         setbmtocaret(tempbm1);			// Start at current cursor location
         chapter = bmchapter(tempbm1);          // Current chapter
         while (!bmnextstory(tempbm) && bmchapter(tempbm) != chapter);     // Get the scan into the correct chapter
         setbmtobm(tempbm, tempbm1);            // Starting location in chapter
         bmdelete(tempbm1);
         Sections_do_sections(tempbm, 0);

         if (bbits == 4) Sections_close(winhandle);
      }
      else
      {
        // There is no known cursor position.

        string s = "{SECTIONS_09}";
        translate(s);
        messagebox(s);
      }
      break;

    case 23:			// Set Values
      Sections_set_options(winhandle);
      Sections_save_options();
      if (bbits == 4) Sections_close(winhandle);
      break;

    case 6:                    // All the menu buttons
    case 7:
    case 8:
    case 9:
      // Open the menu off the menu buttons.
      Sections_openwinmenu (winhandle, icon);
      break;

    case 18:			// Prefix option
      select_icon (winhandle, 18, Sections_PrefixOptTemp = !Sections_PrefixOptTemp);
      if (Sections_PrefixOptTemp)
        set_icon_state (Sections_win, 20, 0x00000000, 0x00400000);
      else
        set_icon_state (Sections_win, 20, 0x00400000, 0x00400000);
      break;

    case 27:			// Automatic option
      select_icon (winhandle, 27, Sections_AutoOptTemp = !Sections_AutoOptTemp);
      if (Sections_AutoOptTemp)
      {
        set_icon_state (Sections_win, 28, 0x00400000, 0x00400000);
        set_icon_state (Sections_win, 35, 0x00400000, 0x00400000);
      }
      else
      {
        set_icon_state (Sections_win, 28, 0x00000000, 0x00400000);
        if (Sections_AllChapsOptTemp)
          set_icon_state (Sections_win, 35, 0x00000000, 0x00400000);
      }
      break;

    case 31:			// All chapters option
      select_icon (winhandle, 31, Sections_AllChapsOptTemp = !Sections_AllChapsOptTemp);
      if (Sections_AllChapsOptTemp && !Sections_AutoOptTemp)
        set_icon_state (Sections_win, 35, 0x00000000, 0x00400000);
      else
        set_icon_state (Sections_win, 35, 0x00400000, 0x00400000);
      break;

    case 34:                    // Do . after level 1 heading option
      select_icon (winhandle, 34, Sections_PeriodOptTemp = !Sections_PeriodOptTemp);
      break;

    case 35:                    // Don't reset the level 1 section number for a new chapter
      select_icon (winhandle, 35, Sections_ContNumOptTemp = !Sections_ContNumOptTemp);
      break;

    case 24:                    // Arabic
      select_icon(winhandle, 24, 1);
      select_icon(winhandle, 25, 0);
      select_icon(winhandle, 26, 0);
      Sections_NumberFormatTemp = 2;
      break;

    case 25:                    // Roman lower
      select_icon(winhandle, 25, 1);
      select_icon(winhandle, 24, 0);
      select_icon(winhandle, 26, 0);
      Sections_NumberFormatTemp = 1;
      break;

    case 26:                    // Roman upper
      select_icon(winhandle, 26, 1);
      select_icon(winhandle, 24, 0);
      select_icon(winhandle, 25, 0);
      Sections_NumberFormatTemp = 0;
      break;

    default:                   // click anywhere else - don't do anything
      break;

  }
}

//---------------------------------------------------------------------------------------------

// Key press event handler for sections style window.

int Sections_key_press (int winhandle, int icon, int key)
{
  switch (key)
  {
    case 13:			// Return, simulate mouse click over Do Sections
      Sections_click (winhandle, 1, 4, 0, 0);
      break;

    case 27:			// Esc, simulate mouse click over Cancel
      Sections_click (winhandle, 2, 4, 0, 0);
      break;

    default:
      break;
   }

   return key;
}

//=============================================================================================

////////////////////////////////////////////////
// Sections Menu Functions                    //
////////////////////////////////////////////////

// Called when the Sections menu entry is clicked.

int Sections_command (int n, int sub)
{

  Sections_close(Sections_win);


  // Check if the file has changed.

  if (Sections_current_file != currentfile())
  {
    Sections_current_file = currentfile();	// Get file handle
    Sections_load_document ();			// Load options saved with document
  }


  Sections_current_view=currentview();

  // Add handlers for when the user clicks on the sections styles window or when
  // s(he) hits a key with it displayed.

  Sections_win = create_windowparent("SectionsEdit",viewhandle(currentview()));

  addwindowhandler(0, Sections_win, "Sections_click");
  addwindowhandler(1, Sections_win, "Sections_key_press");
  addwindowhandler(7, Sections_win, "Sections_close");
  addwindowhandler(2, Sections_win, "Sections_close");
 
  
  Sections_StyleWin();				// Display the Sections window

  setactive(Sections_win);
}

//=============================================================================================

////////////////////////////////////////////////
// Event handlers.			      //
////////////////////////////////////////////////

// Called when a file is closed.

void Sections_file_closing (int user, int file)
{
  if (file == Sections_current_file)
  {
    Sections_close(Sections_win);

  }
}

//---------------------------------------------------------------------------------------------

// Called when a view is made active.

void Sections_view_active (int user, int view, int gain)
{
 if(view == Sections_current_view)
  {
    if (!gain)
    {
     Sections_close(Sections_win);
    }
  }
}

//---------------------------------------------------------------------------------------------

// Called when a view is closed.

void Sections_view_closed (int user, int view)
{
  if (view == Sections_current_view)
  {
   Sections_close(Sections_win);
  }
}

//=============================================================================================

////////////////////////////////////////////////
// Main Functions                             //
////////////////////////////////////////////////

void main(void)
{
  string help = "{SECTIONS_10}";

  Sections_load_defaults ();	// Set the default options

  // Create the menus and windows, assign to handles.

  Sections_stylemenu = create_menu("{SECTIONS_05}");


  // Add sections menu to applets menu.

  script_menu_initialise();             // Initialise the menu
  script_menu_insert("Sections_command", "", "", "", "{SECTIONS_00}");


  // Add handlers for files and views being closed and views made active.

  addeventhandler (0x02, 0, "Sections_file_closing");
  addeventhandler (0x06, 0, "Sections_view_closed");
  addeventhandler (0x215, 0, "Sections_view_active");

  // Define a macro to launch us off.

  if (isdefmacro("sections") == -1)
  {
    translate(help);
    defmacro3(0,"sections", "{Sections_command(0,0)}", "sections", help, "Sections");
  }
}
